@@ -1,4 +1,4 @@
-from .test_gaussian_parser import *
-from .test_base_parser import *
 from .test_extraction import *
+from .test_gaussian_parser import *
+from .test_parser_base import *
 from .test_soxhlet import *
